

local XLog = XLog
local XLog_Events = XLog:NewModule("XLogEventHandler", "LibRockEvent-1.0", "LibXEvent-1.0", "LibXTags-1.0")


--local self = XLog_Events

XLog_Events.db = XLog:GetDatabaseNamespace("XLog")


local selfPlayerName


function XLog_Events:OnEnable()

	-- register as handler
	self:RegisterHandler(XLog_Events.EventHandler)

	selfPlayerName = UnitName("player")
  
end


-- ********************
-- COMBAT LOG EVENTS
	
function XLog_Events:EventHandler(info)

  local self = XLog_Events
		
	-- check incoming/outgoing and self/other
	--local selfOther, inOut = SelfOther_InOut(info.sourceName, info.destName)
	local selfOther = ""
	if info.self then
	  selfOther = "self"
	else
	  selfOther = "other"
	end
		
	-- get event settings root
	local status, eventSettings = pcall(
		function () 
			return self.db.profile["events"][selfOther][info.direction][info.event]
		end
	)
	
	-- if we have eventSettings, and we're tracking this event, continue
	if (eventSettings ~= nil) then
	  if (eventSettings.track) then
			XLog_Events:ProcessEvent(eventSettings, selfOther, info)
		end	  
	end
end


-- Process a valid, tracked event.
function XLog_Events:ProcessEvent(eventSettings, selfOther, info)

  local self = XLog_Events
				
	-- replace the spell school text
	-- NOTE: this is slow, is there a better way to handle this?
	if info.spellSchool ~= nil then
		info.spellSchool = self.db.profile["settings"]["spellSchools"][info.spellSchool]
	end			
	if info.school ~= nil then
		info.school = self.db.profile["settings"]["spellSchools"][info.school]
	end
		
	
	-- if player is target and source, replace target with "you"
	if (info.destName == selfPlayerName and info.sourceName == selfPlayerName) then
		info.destName = self.db.profile["settings"]["you"]
	end
	
	-- get the correct event section
	if (info.auraType ~= nil) then
		-- aura
		eventSettings = eventSettings[info.auraType]
	else
		if (info.critical == 1) then
			eventSettings = eventSettings["crit"]
			if eventSettings == nil then
				message("ERROR: Could not find crit settings for " .. event)
			end
		else
			eventSettings = eventSettings["normal"]
		end
	end
	
	-- format the message
	-- calls LibXTags:GetString
	local msg = XLog_Events:GetString(info, eventSettings.text)
	
	
	-- get the index
	local index = XLog_Events:GetIndex(eventSettings.index, selfOther, info.direction)
	
	-- display
	XLog_Events:ShowLogMessage(msg, eventSettings.color, index)
end

-- /COMBAT LOG EVENTS
-- ******************


-- get the node or parent chat index.
function XLog_Events:GetIndex(eventIndex, selfOther, inOut)
  
  local self = XLog_Events

	local index = eventIndex
	
	if ((index == "-1" or index == -1) and (selfOther ~= nil)) then
		index = self.db.profile["events"][selfOther][inOut]["defaults"].index
	end
	
	-- get root index if necessary
	if (index == "-1" or index == -1) then
		index = self.db.profile["events"]["defaults"].index
	end
		
	return index
end


-- display it.
function XLog_Events:ShowLogMessage(msg, col, index)
  	local win = getglobal("ChatFrame" .. index);
  	win:AddMessage(msg, XLog:GetColor(col))
end