

local XLog = XLog
local XLog_Options = XLog:NewModule("XLogOptions")
local self = XLog_Options

local tabList = {}
local rootTabList = {}

self.db = XLog:GetDatabaseNamespace("XLog")

XLog:SetDatabaseNamespaceDefaults("XLog", "profile", {
    settings = {
        spellSchools = {
            [0] = "None",
            [1] = "Physical",
            [2] = "Holy",
            [4] = "Fire",
            [8] = "Nature",
            [16] = "Frost",
            [32] = "Shadow",
            [64] = "Arcane",
        },
        you = "you",
    },
    events = {
        defaults = {
            index = "2",
        },
        combat = {
        	["track"] = true,
			ENTER_COMBAT = {
				["text"] = "-- Combat Starts --",
				["color"] = {["R"]=1.0, ["G"]=1.0, ["B"]=1.0},
				["index"] = "-1",
			},
			EXIT_COMBAT = {
				["text"] = "-- Combat Ends --",
				["color"] = {["R"]=1.0, ["G"]=1.0, ["B"]=1.0},
				["index"] = "-1",
			}
		},
		gain = {
			HONOR_GAIN = {
				["track"] = true,
				["text"] = "+ [Honor]",
				["color"] = {["R"]=0.0, ["G"]=0.0, ["B"]=1.0},
				["index"] = "-1",
			},
			XP_GAIN = {
				["track"] = true,
				["text"] = "+ [XP]",
				["color"] = {["R"]=0.4, ["G"]=0.0, ["B"]=1.0},
				["index"] = "-1",
			},
		},
        self = {
          -- OUTGOING EVENTS (You vs Other)
            outgoing = {
                defaults = {
                    index = "-1",
                },
              -- SWINGS
                SWING_DAMAGE = {
                    ["track"] = true,
                    normal = {
                        ["text"] = "<[Amount]> on [Target] ([School])",
                        ["color"] = {["R"]=1.0, ["G"]=1.0, ["B"]=1.0},
                        ["index"] = "-1",
                    },
                    crit = {
                        ["text"] = "*[Amount]* on [Target] ([School])",
                        ["color"] = {["R"]=1.0, ["G"]=1.0, ["B"]=1.0},
                        ["index"] = "-1",
                    }
                },            
                SWING_MISSED = {
                ["track"] = true,
                    normal = {
                        ["text"] = "[MissType] Swing on [Target]",
                        ["color"] = {["R"]=1.0, ["G"]=1.0, ["B"]=1.0},
                        ["index"] = "-1",
                    }
                },
              
              
                -- RANGED ATTACKS
                RANGE_DAMAGE = {
                    ["track"] = true,
                    normal = {
                        ["text"] = "<[Amount]> [Spell] on [Target] ([School])",
                        ["color"] = {["R"]=1.0, ["G"]=1.0, ["B"]=1.0},
                        ["index"] = "-1",
                    },
                    crit = {
                        ["text"] = "*[Amount]* [Spell] on [Target] ([School])",
                        ["color"] = {["R"]=1.0, ["G"]=1.0, ["B"]=1.0},
                        ["index"] = "-1",
                    }
                },
                RANGE_MISSED = {
                    ["track"] = true,
                    normal = {
						["text"] = "[MissType] [Spell] on [Target]",
						["color"] = {["R"]=1.0, ["G"]=1.0, ["B"]=1.0},
						["index"] = "-1",
                    }
                },
                  
                  
                  -- SPELL ATTACKS
                SPELL_DAMAGE = {
                    ["track"] = true,
                    normal = {
                        ["text"] = "<[Amount]> [Spell] on [Target] ([School])",
                        ["color"] = {["R"]=1.0, ["G"]=1.0, ["B"]=0},
                        ["index"] = "-1",
                    },
                    crit = {
                        ["text"] = "*[Amount]* [Spell] on [Target] ([School])",
                        ["color"] = {["R"]=1.0, ["G"]=1.0, ["B"]=0},
                        ["index"] = "-1",
                    },
                },          
                SPELL_MISSED = {
                    ["track"] = true,
                    normal = {
                        ["text"] = "[MissType] [Spell] on [Target]",
                        ["color"] = {["R"]=1.0, ["G"]=1.0, ["B"]=0},
                        ["index"] = "-1",
                    }
                },
                
                
                -- SPELL PERIODIC ATTACKS
                SPELL_PERIODIC_DAMAGE = {
                    ["track"] = true,
                    normal = {
                        ["text"] = "<[Amount]> [Spell] on [Target] ([School])",
                        ["color"] = {["R"]=1.0, ["G"]=1.0, ["B"]=0},
                        ["index"] = "-1",
                    }
                },          
                SPELL_PERIODIC_MISSED = {
                    ["track"] = true,
                    normal = {
                        ["text"] = "[MissType] [Spell] on [Target]",
                        ["color"] = {["R"]=1.0, ["G"]=1.0, ["B"]=0},
                        ["index"] = "-1",
                    },
                },
            
        
				-- SPELL HEALS
				SPELL_HEAL = {
					["track"] = true,
					normal = {
						["text"] = "<[Amount]> Your [Spell] heals [Target]",
						["color"] = {["R"]=0.0, ["G"]=0.8, ["B"]=0.0},
						["index"] = "-1",
					},
					crit = {
						["text"] = "*[Amount]* Your [Spell] heals [Target]",
						["color"] = {["R"]=0.0, ["G"]=0.8, ["B"]=0.0},
						["index"] = "-1",
					}
				},
            
            
            	-- SPELL PERIODIC HEALS
				SPELL_PERIODIC_HEAL = {
					["track"] = true,
					normal = {
						["text"] = "<[Amount]> Your [Spell] heals [Target]",
						["color"] = {["R"]=0.0, ["G"]=0.8, ["B"]=0.0},
						["index"] = "-1",
					},
				},
				
				SPELL_AURA_APPLIED = {
					["track"] = true,
					BUFF = {
						["text"] = "++[Spell]",
						["color"] = {["R"]=1.0, ["G"]=1.0, ["B"]=1.0},
						["index"] = "-1",
					},
					DEBUFF = {
						["text"] = "++[Spell]",
						["color"] = {["R"]=1.0, ["G"]=1.0, ["B"]=1.0},
						["index"] = "-1",
					}
				},
				SPELL_AURA_REMOVED = {
					["track"] = true,
					BUFF = {
						["text"] = "--[Spell]",
						["color"] = {["R"]=0.0, ["G"]=1.0, ["B"]=1.0},
						["index"] = "-1",
					},
					DEBUFF = {
						["text"] = "--[Spell]",
						["color"] = {["R"]=1.0, ["G"]=0.0, ["B"]=1.0},
						["index"] = "-1",
					}
				},				
            },
            -- END OF OUTGOING EVENTS
            
            
            -- INCOMING EVENTS (Other vs You)
            incoming = {
                defaults = {
                    index = "-1",
                },
              -- SWING ATTACKS
                SWING_DAMAGE = {
                    ["track"] = true,
                    normal = {
                        ["text"] = "<[Amount]> [Source] ([School])",
                        ["color"] = {["R"]=0.8, ["G"]=0.0, ["B"]=0},
                        ["index"] = "-1",
                    },
                    crit = {
                        ["text"] = "*[Amount]* [Source] ([School])",
                        ["color"] = {["R"]=0.8, ["G"]=0.0, ["B"]=0},
                        ["index"] = "-1",
                    },
                },            
                SWING_MISSED = {
                    ["track"] = true,
                    normal = {
                        ["text"] = "[MissType] Swing from [Source]",
                        ["color"] = {["R"]=0.8, ["G"]=0.8, ["B"]=0.8},
                        ["index"] = "-1",
                    }
                },
              
              
              -- RANGE ATTACKS
                RANGE_DAMAGE = {
                    ["track"] = true,
                    normal = {
                        ["text"] = "<[Amount]> [Source] with [Spell] ([School])",
                        ["color"] = {["R"]=0.8, ["G"]=0.0, ["B"]=0},
                        ["index"] = "-1",
                    },
                    crit = {
                        ["text"] = "*[Amount]* [Source] with [Spell] ([School])",
                        ["color"] = {["R"]=0.8, ["G"]=0.0, ["B"]=0},
                        ["index"] = "-1",
                    },
                },            
                RANGE_MISSED = {
                    ["track"] = true,
                    normal = {
                        ["text"] = "[MissType] [Spell] from [Source]",
                        ["color"] = {["R"]=0.8, ["G"]=0.8, ["B"]=0.8},
                        ["index"] = "-1",
                    }
                },
              
              
              	--- SPELL ATTACKS
				SPELL_DAMAGE = {
					["track"] = true,
					normal = {
						["text"] = "<[Amount]> [Source] with [Spell] ([School])",
						["color"] = {["R"]=0.8, ["G"]=0.0, ["B"]=0.8},
						["index"] = "-1",
					},
					crit = {
						["text"] = "*[Amount]* [Source] with [Spell] ([School])",
						["color"] = {["R"]=0.8, ["G"]=0.0, ["B"]=0.8},
						["index"] = "-1",
					},
				},          
				SPELL_MISSED = {
					["track"] = true,
					normal = {
						["text"] = "[MissType] [Spell] from [Source]",
						["color"] = {["R"]=0.8, ["G"]=0.8, ["B"]=0.8},
						["index"] = "-1",
					}
				},
              
              
				-- SPELL PERIODIC ATTACKS
				SPELL_PERIODIC_DAMAGE = {
					["track"] = true,
					normal = {
						["text"] = "<[Amount]> [Source] with [Spell] ([School])",
						["color"] = {["R"]=0.8, ["G"]=0.0, ["B"]=0.8},
						["index"] = "-1",
					},
					crit = {
						["text"] = "*[Amount]* [Source] with [Spell] ([School])",
						["color"] = {["R"]=0.8, ["G"]=0.0, ["B"]=0.8},
						["index"] = "-1",
					},
				},
				
				SPELL_PERIODIC_MISSED = {
					["track"] = true,
					normal = {
						["text"] = "[MissType] [Spell] from [Source]",
						["color"] = {["R"]=0.8, ["G"]=0.8, ["B"]=0.8},
						["index"] = "-1",
					}
				},

              
				-- SPELL HEALS
				SPELL_HEAL = {
					["track"] = true,
					normal = {
						["text"] = "<[Amount]> [Source]'s [Spell] heals you",
						["color"] = {["R"]=0.0, ["G"]=0.8, ["B"]=0.0},
						["index"] = "-1",
					},
					crit = {
						["text"] = "*[Amount]* [Source]'s [Spell] heals you",
						["color"] = {["R"]=0.0, ["G"]=0.8, ["B"]=0.0},
						["index"] = "-1",
					},
				},
            
            
				-- SPELL PERIODIC HEALS
				SPELL_PERIODIC_HEAL = {
					["track"] = true,
					normal = {
						["text"] = "<[Amount]> [Source]'s [Spell] heals you",
						["color"] = {["R"]=0.0, ["G"]=0.8, ["B"]=0.0},
						["index"] = "-1",
					},
				},
				
				SPELL_AURA_APPLIED = {
					["track"] = true,
					BUFF = {
						["text"] = "++[Spell]",
						["color"] = {["R"]=0.0, ["G"]=1.0, ["B"]=1.0},
						["index"] = "-1",
					},
					DEBUFF = {
						["text"] = "++[Spell]",
						["color"] = {["R"]=1.0, ["G"]=0.0, ["B"]=1.0},
						["index"] = "-1",
					}
				},
				SPELL_AURA_REMOVED = {
					["track"] = true,
					BUFF = {
						["text"] = "--[Spell]",
						["color"] = {["R"]=0.0, ["G"]=1.0, ["B"]=1.0},
						["index"] = "-1",
					},
					DEBUFF = {
						["text"] = "--[Spell]",
						["color"] = {["R"]=1.0, ["G"]=0.0, ["B"]=1.0},
						["index"] = "-1",
					}
				},
				              
            --END OF INCOMING EVENTS
            }
        }
    }
})




function XLog_Options:OnEnable()
        
    XLog:Print("Options Enabled")
    
    -- setup tablist
    tabList = {
	    ["-1"] = " Inherit",
	    ["1"] = ChatName(1),
	    ["2"] = ChatName(2),
	    ["3"] = ChatName(3),
	    ["4"] = ChatName(4),
	    ["5"] = ChatName(5),
	    ["6"] = ChatName(6),
	    ["7"] = ChatName(7),
	    ["8"] = ChatName(8),
	    ["9"] = ChatName(9),
	    ["10"] = ChatName(10)
	}
	
	rootTabList = {
	    ["1"] = ChatName(1),
	    ["2"] = ChatName(2),
	    ["3"] = ChatName(3),
	    ["4"] = ChatName(4),
	    ["5"] = ChatName(5),
	    ["6"] = ChatName(6),
	    ["7"] = ChatName(7),
	    ["8"] = ChatName(8),
	    ["9"] = ChatName(9),
	    ["10"] = ChatName(10)
	}
    
    self:Init()
    
end

function XLog_Options:Init()
    XLog:Print("Init_Log_Options")
    
    local eventsDB = self.db.profile["events"]
    
    local outgoing = {
        name = "Outgoing",
        desc = "Outgoing Events",
        type = "group",
        --icon
        args = {
            -- Root outgoing options
            defaultTab = {
                name = "Default Tab",
                desc = "Default tab to display outgoing messages which do not have a tab chosen.",
                type = 'choice',
                choices = tabList,
                order = 1,
                get = function(key)
                    return eventsDB["self"]["outgoing"]["defaults"]["index"]
                end,
                set = function(key)
                    eventsDB["self"]["outgoing"]["defaults"]["index"] = key
                end,
            },
            trackDirectHeals = {
                name = "Track Heals",
                desc = "Enable/disable heal tracking",
                type = "boolean",
                order = 2,
                get = function()
                        return eventsDB["self"]["outgoing"]["SPELL_HEAL"]["track"]
                    end,
                set = function(value)
                        eventsDB["self"]["outgoing"]["SPELL_HEAL"]["track"] = value
                    end,
            },
            trackHots = {
                name = "Track HoTs",
                desc = "Enable/disable heal over time tracking.",
                type = "boolean",
                order = 3,
                get = function()
                        return eventsDB["self"]["outgoing"]["SPELL_PERIODIC_HEAL"]["track"]
                    end,
                set = function(value)
                        eventsDB["self"]["outgoing"]["SPELL_PERIODIC_HEAL"]["track"] = value
                    end,
            },
            trackSwingHits = {
                name = "Track Swing Hits",
                desc = "Track swing hits",
                type = "boolean",
                order = 10,
                get = function() 
                        return eventsDB["self"]["outgoing"]["SWING_DAMAGE"]["track"]
                    end,
                set = function(value)
                        eventsDB["self"]["outgoing"]["SWING_DAMAGE"]["track"] = value
                    end,
            },
            trackFailedSwings = {
                name = "Track Failed Swings",
                desc = "Track failed swings",
                type = "boolean",
                order = 11,
                get = function() 
                        return eventsDB["self"]["outgoing"]["SWING_MISSED"]["track"]
                    end,
                set = function(value) 
                        eventsDB["self"]["outgoing"]["SWING_MISSED"]["track"] = value
                    end,
            },          
            trackSpellHits = {
                name = "Track Spell Hits",
                desc = "Track spell hits",
                type = "boolean",
                order = 20,
                get = function() 
                        return eventsDB["self"]["outgoing"]["SPELL_DAMAGE"]["track"]
                    end,
                set = function(value)
                        eventsDB["self"]["outgoing"]["SPELL_DAMAGE"]["track"] = value
                    end,
            },
            trackSpellFailed = {
                name = "Track Failed Spells",
                desc = "Track failed spells",
                type = "boolean",
                order = 21,
                get = function() 
                        return eventsDB["self"]["outgoing"]["SPELL_MISSED"]["track"]
                    end,
                set = function(value)
                        eventsDB["self"]["outgoing"]["SPELL_MISSED"]["track"] = value
                    end,
            },
            trackDots = {
            	name = "Track DoTs",
            	desc = "Track damage over time spells.",
            	type = "boolean",
            	order = 30,
            	get = function()
            			return eventsDB["self"]["outgoing"]["SPELL_PERIODIC_DAMAGE"]["track"]
            		end,
            	set = function(value)
            			eventsDB["self"]["outgoing"]["SPELL_PERIODIC_DAMAGE"]["track"] = value
            		end,
            },
            trackFailedDots = {
            	name = "Track Failed DoTs",
            	desc = "Track damage over time spells.",
            	type = "boolean",
            	order = 30,
            	get = function()
            			return eventsDB["self"]["outgoing"]["SPELL_PERIODIC_MISSED"]["track"]
            		end,
            	set = function(value)
            			eventsDB["self"]["outgoing"]["SPELL_PERIODIC_MISSED"]["track"] = value
            		end,
            },
            -- End Root Options
            
            -- Outgoing Heal options
            heals = {
                name = "Heals",
                desc = "Outgoing heals",
                type = "group",
                args = {
                    normal = {
                        name = "Normal",
                        desc = "Normal heals.",
                        type = "group",
                        order = 1,
                        args = {
                            ["color"] = {
                                name = "Color",
                                desc = "Outgoing normal heal color",
                                type = "color",
                                get = 
                                    function() 
                                        return XLog:GetColor(eventsDB["self"]["outgoing"]["SPELL_HEAL"]["normal"]["color"])
                                    end,
                                set = 
                                    function(r,g,b)
                                        local db = eventsDB["self"]["outgoing"]["SPELL_HEAL"]["normal"]["color"]
                                        db["R"], db["G"], db["B"] = r, g, b
                                    end
                            },
                            ["text"] = {
                                name = "Text",
                                desc = "Outgoing swing text",
                                type = "string",
                                usage = "<string>",
                                get = 
                                    function() 
                                        return eventsDB["self"]["outgoing"]["SPELL_HEAL"]["normal"]["text"]
                                    end,
                                set = 
                                    function(value)
                                        eventsDB["self"]["outgoing"]["SPELL_HEAL"]["normal"]["text"] = value
                                    end,
                            },
                            ["tab"] = {
                                name = "Chat Tab",
                                desc = "Chat tab to display messages in.",
                                type = 'choice',
                                choices = tabList,
                                get = function(key)
                                    return eventsDB["self"]["outgoing"]["SPELL_HEAL"]["normal"]["index"]
                                end,
                                set = function(key)
                                    eventsDB["self"]["outgoing"]["SPELL_HEAL"]["normal"]["index"] = key
                                end,
                            }
                        }
                    },
                    crits = {
                        name = "Critical Heals",
                        desc = "Critical Heals",
                        type = "group",
                        hidden = false,
                        args = {
                            ["color"] = {
                                name = "Color",
                                desc = "Outgoing critical heal color",
                                type = "color",
                                get = 
                                    function() 
                                        return XLog:GetColor(eventsDB["self"]["outgoing"]["SPELL_HEAL"]["crit"]["color"])
                                    end,
                                set = 
                                    function(r, g, b) 
                                        local db = eventsDB["self"]["outgoing"]["SPELL_HEAL"]["crit"]["color"]
                                        db["R"], db["G"], db["B"] = r, g, b
                                    end
                            },
                            ["text"] = {
                                name = "Text",
                                desc = "Outgoing critical heal text",
                                type = "string",
                                usage = "<string>",
                                get = 
                                    function() 
                                        return eventsDB["self"]["outgoing"]["SPELL_HEAL"]["crit"]["text"]
                                    end,
                                set = 
                                    function(value)
                                        eventsDB["self"]["outgoing"]["SPELL_HEAL"]["crit"]["text"] = value
                                    end,
                            },
                            ["tab"] = {
                                name = "Chat Tab",
                                desc = "Chat tab to display messages in.",
                                type = 'choice',
                                choices = tabList,
                                get = function(key)
                                    return eventsDB["self"]["outgoing"]["SPELL_HEAL"]["crit"]["index"]
                                end,
                                set = function(key)
                                    eventsDB["self"]["outgoing"]["SPELL_HEAL"]["crit"]["index"] = key
                                end,
                            }
                        }
                    },
                    hots = {
                        name = "HoTs",
                        desc = "Heals over time",
                        type = "group",
                        hidden = false,
                        args = {
                            ["color"] = {
                                name = "Color",
                                desc = "Outgoing HoT color",
                                type = "color",
                                get = 
                                    function() 
                                        return XLog:GetColor(eventsDB["self"]["outgoing"]["SPELL_PERIODIC_HEAL"]["normal"]["color"])
                                    end,
                                set = 
                                    function(r, g, b) 
                                        local db = eventsDB["self"]["outgoing"]["SPELL_PERIODIC_HEAL"]["normal"]["color"]
                                        db["R"], db["G"], db["B"] = r, g, b
                                    end
                            },
                            ["text"] = {
                                name = "Text",
                                desc = "Outgoing HoT text",
                                type = "string",
                                usage = "<string>",
                                get = 
                                    function() 
                                        return eventsDB["self"]["outgoing"]["SPELL_PERIODIC_HEAL"]["normal"]["text"]
                                    end,
                                set = 
                                    function(value)
                                        eventsDB["self"]["outgoing"]["SPELL_PERIODIC_HEAL"]["normal"]["text"] = value
                                    end,
                            },
                            ["tab"] = {
                                name = "Chat Tab",
                                desc = "Chat tab to display messages in.",
                                type = 'choice',
                                choices = tabList,
                                get = function(key)
                                    return eventsDB["self"]["outgoing"]["SPELL_PERIODIC_HEAL"]["normal"]["index"]
                                end,
                                set = function(key)
                                    eventsDB["self"]["outgoing"]["SPELL_PERIODIC_HEAL"]["normal"]["index"] = key
                                end,
                            }
                        }
                    },                  
                }
            },
            -- End heal options
            
            -- Outgoing Swing Options
            swings = {
                name = "Swings",
                desc = "White damage",
                type = "group",
                args = {
                    normal = {
                        name = "Normal",
                        desc = "Swing hits",
                        type = "group",
                        order = 1,
                        args = {
                            ["color"] = {
                                name = "Color",
                                desc = "Outgoing swing color",
                                type = "color",
                                get = 
                                    function() 
                                        return XLog:GetColor(eventsDB["self"]["outgoing"]["SWING_DAMAGE"]["normal"]["color"])
                                    end,
                                set = 
                                    function(r,g,b)
                                        local db = eventsDB["self"]["outgoing"]["SWING_DAMAGE"]["normal"]["color"]
                                        db["R"], db["G"], db["B"] = r, g, b
                                    end
                            },
                            ["text"] = {
                                name = "Text",
                                desc = "Outgoing swing text",
                                type = "string",
                                usage = "<string>",
                                get = 
                                    function() 
                                        return eventsDB["self"]["outgoing"]["SWING_DAMAGE"]["normal"]["text"]
                                    end,
                                set = 
                                    function(value)
                                        eventsDB["self"]["outgoing"]["SWING_DAMAGE"]["normal"]["text"] = value
                                    end,
                            },
                            ["tab"] = {
                                name = "Chat Tab",
                                desc = "Chat tab to display messages in.",
                                type = 'choice',
                                choices = tabList,
                                get = function(key)
                                    --message(ChatName(eventsDB["self"]["outgoing"]["SWING_DAMAGE"]["tabIndex"]))
                                    return eventsDB["self"]["outgoing"]["SWING_DAMAGE"]["normal"]["index"]
                                end,
                                set = function(key)
                                    --message(key)
                                    eventsDB["self"]["outgoing"]["SWING_DAMAGE"]["normal"]["index"] = key
                                end,
                            }
                        }
                    },
                    crits = {
                        name = "Critical Strikes",
                        desc = "Critical Strikes",
                        type = "group",
                        hidden = false,
                        args = {
                            ["color"] = {
                                name = "Color",
                                desc = "Outgoing critical swing color",
                                type = "color",
                                get = 
                                    function() 
                                        return XLog:GetColor(eventsDB["self"]["outgoing"]["SWING_DAMAGE"]["crit"]["color"])
                                    end,
                                set = 
                                    function(r, g, b) 
                                        local db = eventsDB["self"]["outgoing"]["SWING_DAMAGE"]["crit"]["color"]
                                        db["R"], db["G"], db["B"] = r, g, b
                                    end
                            },
                            ["text"] = {
                                name = "Text",
                                desc = "Outgoing critical swing text",
                                type = "string",
                                usage = "<string>",
                                get = 
                                    function() 
                                        return eventsDB["self"]["outgoing"]["SWING_DAMAGE"]["crit"]["text"]
                                    end,
                                set = 
                                    function(value)
                                        eventsDB["self"]["outgoing"]["SWING_DAMAGE"]["crit"]["text"] = value
                                    end,
                            },
                            ["tab"] = {
                                name = "Chat Tab",
                                desc = "Chat tab to display messages in.",
                                type = 'choice',
                                choices = tabList,
                                get = function(key)
                                    return eventsDB["self"]["outgoing"]["SWING_DAMAGE"]["crit"]["index"]
                                end,
                                set = function(key)
                                    eventsDB["self"]["outgoing"]["SWING_DAMAGE"]["crit"]["index"] = key
                                end,
                            }
                        }
                    },
                    miss = {
                        name = "Failed Attacks",
                        desc = "Failed to hit swings",
                        type = "group",
                        hidden = false,
                        args = {
                            ["color"] = {
                                name = "Color",
                                desc = "Outgoing failed swing color",
                                type = "color",
                                get = 
                                    function() 
                                        return XLog:GetColor(eventsDB["self"]["outgoing"]["SWING_MISSED"]["normal"]["color"])
                                    end,
                                set = 
                                    function(r, g, b)
                                        local db = eventsDB["self"]["outgoing"]["SWING_MISSED"]["normal"]["color"]
                                        db["R"], db["G"], db["B"] = r, g, b
                                    end
                            },
                            ["text"] = {
                                name = "Text",
                                desc = "Outgoing failed swing text",
                                type = "string",
                                usage = "<string>",
                                get = 
                                    function() 
                                        return eventsDB["self"]["outgoing"]["SWING_MISSED"]["normal"]["text"]
                                    end,
                                set = 
                                    function(value)
                                        eventsDB["self"]["outgoing"]["SWING_MISSED"]["normal"]["text"] = value
                                    end,
                            },
                            ["tab"] = {
                                name = "Chat Tab",
                                desc = "Chat tab to display messages in.",
                                type = 'choice',
                                choices = tabList,
                                get = function(key)
                                    return eventsDB["self"]["outgoing"]["SWING_MISSED"]["normal"]["index"]
                                end,
                                set = function(key)
                                    eventsDB["self"]["outgoing"]["SWING_MISSED"]["normal"]["index"] = key
                                end,
                            }                           
                        }
                    }                   
                }
            },
            -- End swing options
            
            -- Outgoing spell options
            spells = {
                name = "Spells",
                desc = "Spell/Special damage",
                type = "group",
                args = {
                    normal = {
                        name = "Normal",
                        desc = "Spell hits",
                        type = "group",
                        order = 1,
                        args = {
                            ["color"] = {
                                name = "Color",
                                desc = "Outgoing spell color",
                                type = "color",
                                get = 
                                    function() 
                                        return XLog:GetColor(eventsDB["self"]["outgoing"]["SPELL_DAMAGE"]["normal"]["color"])
                                    end,
                                set = 
                                    function(r,g,b)
                                        local db = eventsDB["self"]["outgoing"]["SPELL_DAMAGE"]["normal"]["color"]
                                        db["R"], db["G"], db["B"] = r, g, b
                                    end
                            },
                            ["text"] = {
                                name = "Text",
                                desc = "Outgoing spell text",
                                type = "string",
                                usage = "<string>",
                                get = 
                                    function() 
                                        return eventsDB["self"]["outgoing"]["SPELL_DAMAGE"]["normal"]["text"]
                                    end,
                                set = 
                                    function(value)
                                        eventsDB["self"]["outgoing"]["SPELL_DAMAGE"]["normal"]["text"] = value
                                    end,
                            },
                            ["tab"] = {
                                name = "Chat Tab",
                                desc = "Chat tab to display messages in.",
                                type = 'choice',
                                choices = tabList,
                                get = function(key)
                                    return eventsDB["self"]["outgoing"]["SPELL_DAMAGE"]["normal"]["index"]
                                end,
                                set = function(key)
                                    eventsDB["self"]["outgoing"]["SPELL_DAMAGE"]["normal"]["index"] = key
                                end,
                            }
                        }
                    },
                    crits = {
                        name = "Critical Strikes",
                        desc = "Critical Strikes",
                        type = "group",
                        hidden = false,
                        args = {
                            ["color"] = {
                                name = "Color",
                                desc = "Outgoing critical spell color",
                                type = "color",
                                get = 
                                    function() 
                                        return XLog:GetColor(eventsDB["self"]["outgoing"]["SPELL_DAMAGE"]["crit"]["color"])
                                    end,
                                set = 
                                    function(r, g, b) 
                                        local db = eventsDB["self"]["outgoing"]["SPELL_DAMAGE"]["crit"]["color"]
                                        db["R"], db["G"], db["B"] = r, g, b
                                    end
                            },
                            ["text"] = {
                                name = "Text",
                                desc = "Outgoing critical spell text",
                                type = "string",
                                usage = "<string>",
                                get = 
                                    function() 
                                        return eventsDB["self"]["outgoing"]["SPELL_DAMAGE"]["crit"]["text"]
                                    end,
                                set = 
                                    function(value)
                                        eventsDB["self"]["outgoing"]["SPELL_DAMAGE"]["crit"]["text"] = value
                                    end,
                            },
                            ["tab"] = {
                                name = "Chat Tab",
                                desc = "Chat tab to display messages in.",
                                type = 'choice',
                                choices = tabList,
                                get = function(key)
                                    --message(ChatName(eventsDB["self"]["outgoing"]["SWING_DAMAGE"]["tabIndex"]))
                                    return eventsDB["self"]["outgoing"]["SPELL_DAMAGE"]["crit"]["index"]
                                end,
                                set = function(key)
                                    --message(key)
                                    eventsDB["self"]["outgoing"]["SPELL_DAMAGE"]["crit"]["index"] = key
                                end,
                            }
                        }
                    },
                    miss = {
                        name = "Failed Attacks",
                        desc = "Failed to hit spells",
                        type = "group",
                        hidden = false,
                        args = {
                            ["color"] = {
                                name = "Color",
                                desc = "Outgoing failed spell color",
                                type = "color",
                                get = 
                                    function() 
                                        return XLog:GetColor(eventsDB["self"]["outgoing"]["SPELL_MISSED"]["normal"]["color"])
                                    end,
                                set = 
                                    function(r, g, b)
                                        local db = eventsDB["self"]["outgoing"]["SPELL_MISSED"]["normal"]["color"]
                                        db["R"], db["G"], db["B"] = r, g, b
                                    end
                            },
                            ["text"] = {
                                name = "Text",
                                desc = "Outgoing failed spell text",
                                type = "string",
                                usage = "<string>",
                                get = 
                                    function() 
                                        return eventsDB["self"]["outgoing"]["SPELL_MISSED"]["normal"]["text"]
                                    end,
                                set = 
                                    function(value)
                                        eventsDB["self"]["outgoing"]["SPELL_MISSED"]["normal"]["text"] = value
                                    end,
                            },
                            ["tab"] = {
                                name = "Chat Tab",
                                desc = "Chat tab to display messages in.",
                                type = 'choice',
                                choices = tabList,
                                get = function(key)
                                    return eventsDB["self"]["outgoing"]["SPELL_MISSED"]["normal"]["index"]
                                end,
                                set = function(key)
                                    eventsDB["self"]["outgoing"]["SPELL_MISSED"]["normal"]["index"] = key
                                end,
                            }                           
                        }
                    }                   
                }
            }                       
        }
    }

    local incoming = {
        name = "Incoming",
        desc = "Incoming Events",
        type = "group",
        --icon
        args = {
        	-- Root Options
            ["defaultTab"] = {
                name = "Default Tab",
                desc = "Default tab to display incoming messages which do not have a tab chosen.",
                type = 'choice',
                choices = tabList,
                order = 1,
                get = function(key)
                    return eventsDB["self"]["incoming"]["defaults"]["index"]
                end,
                set = function(key)
                    eventsDB["self"]["incoming"]["defaults"]["index"] = key
                end,
            },
            trackDirectHeals = {
                name = "Track Heals",
                desc = "Enable/disable heal tracking",
                type = "boolean",
                order = 2,
                get = function()
                        return eventsDB["self"]["incoming"]["SPELL_HEAL"]["track"]
                    end,
                set = function(value)
                        eventsDB["self"]["incoming"]["SPELL_HEAL"]["track"] = value
                    end,
            },
            trackHots = {
                name = "Track HoTs",
                desc = "Enable/disable heal over time tracking.",
                type = "boolean",
                order = 3,
                get = function()
                        return eventsDB["self"]["incoming"]["SPELL_PERIODIC_HEAL"]["track"]
                    end,
                set = function(value)
                        eventsDB["self"]["incoming"]["SPELL_PERIODIC_HEAL"]["track"] = value
                    end,
            },
            trackSwingHits = {
                name = "Track Swing Hits",
                desc = "Track swing hits",
                type = "boolean",
                order = 10,
                get = function() 
                        return eventsDB["self"]["incoming"]["SWING_DAMAGE"]["track"]
                    end,
                set = function(value)
                        eventsDB["self"]["incoming"]["SWING_DAMAGE"]["track"] = value
                    end,
            },
            trackFailedSwings = {
                name = "Track Failed Swings",
                desc = "Track failed swings",
                type = "boolean",
                order = 11,
                get = function() 
                        return eventsDB["self"]["incoming"]["SWING_MISSED"]["track"]
                    end,
                set = function(value) 
                        eventsDB["self"]["incoming"]["SWING_MISSED"]["track"] = value
                    end,
            },          
            trackSpellHits = {
                name = "Track Spell Hits",
                desc = "Track spell hits",
                type = "boolean",
                order = 20,
                get = function() 
                        return eventsDB["self"]["incoming"]["SPELL_DAMAGE"]["track"]
                    end,
                set = function(value)
                        eventsDB["self"]["incoming"]["SPELL_DAMAGE"]["track"] = value
                    end,
            },
            trackSpellFailed = {
                name = "Track Failed Spells",
                desc = "Track failed spells",
                type = "boolean",
                order = 21,
                get = function() 
                        return eventsDB["self"]["incoming"]["SPELL_MISSED"]["track"]
                    end,
                set = function(value)
                        eventsDB["self"]["incoming"]["SPELL_MISSED"]["track"] = value
                    end,
            },
            trackDots = {
            	name = "Track DoTs",
            	desc = "Track damage over time spells.",
            	type = "boolean",
            	order = 30,
            	get = function()
            			return eventsDB["self"]["incoming"]["SPELL_PERIODIC_DAMAGE"]["track"]
            		end,
            	set = function(value)
            			eventsDB["self"]["incoming"]["SPELL_PERIODIC_DAMAGE"]["track"] = value
            		end,
            },
            trackFailedDots = {
            	name = "Track Failed DoTs",
            	desc = "Track damage over time spells.",
            	type = "boolean",
            	order = 30,
            	get = function()
            			return eventsDB["self"]["incoming"]["SPELL_PERIODIC_MISSED"]["track"]
            		end,
            	set = function(value)
            			eventsDB["self"]["incoming"]["SPELL_PERIODIC_MISSED"]["track"] = value
            		end,
            },
            -- End Root Options            
            
            heals = {
                name = "Heals",
                desc = "Incoming heals",
                type = "group",
                args = {
                    normal = {
                        name = "Normal",
                        desc = "Normal heals.",
                        type = "group",
                        order = 1,
                        args = {
                            ["color"] = {
                                name = "Color",
                                desc = "Incoming normal heal color",
                                type = "color",
                                get = 
                                    function() 
                                        return XLog:GetColor(eventsDB["self"]["incoming"]["SPELL_HEAL"]["normal"]["color"])
                                    end,
                                set = 
                                    function(r,g,b)
                                        local db = eventsDB["self"]["incoming"]["SPELL_HEAL"]["normal"]["color"]
                                        db["R"], db["G"], db["B"] = r, g, b
                                    end
                            },
                            ["text"] = {
                                name = "Text",
                                desc = "Incoming swing text",
                                type = "string",
                                usage = "<string>",
                                get = 
                                    function() 
                                        return eventsDB["self"]["incoming"]["SPELL_HEAL"]["normal"]["text"]
                                    end,
                                set = 
                                    function(value)
                                        eventsDB["self"]["incoming"]["SPELL_HEAL"]["normal"]["text"] = value
                                    end,
                            },
                            ["tab"] = {
                                name = "Chat Tab",
                                desc = "Chat tab to display messages in.",
                                type = 'choice',
                                choices = tabList,
                                get = function(key)
                                    return eventsDB["self"]["incoming"]["SPELL_HEAL"]["normal"]["index"]
                                end,
                                set = function(key)
                                    eventsDB["self"]["incoming"]["SPELL_HEAL"]["normal"]["index"] = key
                                end,
                            }
                        }
                    },
                    crits = {
                        name = "Critical Heals",
                        desc = "Critical Heals",
                        type = "group",
                        hidden = false,
                        args = {
                            ["color"] = {
                                name = "Color",
                                desc = "Incoming critical heal color",
                                type = "color",
                                get = 
                                    function() 
                                        return XLog:GetColor(eventsDB["self"]["incoming"]["SPELL_HEAL"]["crit"]["color"])
                                    end,
                                set = 
                                    function(r, g, b) 
                                        local db = eventsDB["self"]["incoming"]["SPELL_HEAL"]["crit"]["color"]
                                        db["R"], db["G"], db["B"] = r, g, b
                                    end
                            },
                            ["text"] = {
                                name = "Text",
                                desc = "Incoming critical heal text",
                                type = "string",
                                usage = "<string>",
                                get = 
                                    function() 
                                        return eventsDB["self"]["incoming"]["SPELL_HEAL"]["crit"]["text"]
                                    end,
                                set = 
                                    function(value)
                                        eventsDB["self"]["incoming"]["SPELL_HEAL"]["crit"]["text"] = value
                                    end,
                            },
                            ["tab"] = {
                                name = "Chat Tab",
                                desc = "Chat tab to display messages in.",
                                type = 'choice',
                                choices = tabList,
                                get = function(key)
                                    return eventsDB["self"]["incoming"]["SPELL_HEAL"]["crit"]["index"]
                                end,
                                set = function(key)
                                    eventsDB["self"]["incoming"]["SPELL_HEAL"]["crit"]["index"] = key
                                end,
                            }
                        }
                    },
                    hots = {
                        name = "HoTs",
                        desc = "Heals over time",
                        type = "group",
                        hidden = false,
                        args = {
                            ["color"] = {
                                name = "Color",
                                desc = "Incoming HoT color",
                                type = "color",
                                get = 
                                    function() 
                                        return XLog:GetColor(eventsDB["self"]["incoming"]["SPELL_PERIODIC_HEAL"]["normal"]["color"])
                                    end,
                                set = 
                                    function(r, g, b) 
                                        local db = eventsDB["self"]["incoming"]["SPELL_PERIODIC_HEAL"]["normal"]["color"]
                                        db["R"], db["G"], db["B"] = r, g, b
                                    end
                            },
                            ["text"] = {
                                name = "Text",
                                desc = "Incoming HoT text",
                                type = "string",
                                usage = "<string>",
                                get = 
                                    function() 
                                        return eventsDB["self"]["incoming"]["SPELL_PERIODIC_HEAL"]["normal"]["text"]
                                    end,
                                set = 
                                    function(value)
                                        eventsDB["self"]["incoming"]["SPELL_PERIODIC_HEAL"]["normal"]["text"] = value
                                    end,
                            },
                            ["tab"] = {
                                name = "Chat Tab",
                                desc = "Chat tab to display messages in.",
                                type = 'choice',
                                choices = tabList,
                                get = function(key)
                                    return eventsDB["self"]["incoming"]["SPELL_PERIODIC_HEAL"]["normal"]["index"]
                                end,
                                set = function(key)
                                    eventsDB["self"]["incoming"]["SPELL_PERIODIC_HEAL"]["normal"]["index"] = key
                                end,
                            }
                        }
                    },                  
                }
            },
            swings = {
                name = "Swings",
                desc = "White damage",
                type = "group",
                args = {
                    trackHits = {
                        name = "Track Hits",
                        desc = "Track swing hits",
                        type = "boolean",
                        get = 
                            function() 
                                return eventsDB["self"]["incoming"]["SWING_DAMAGE"]["track"]
                            end,
                        set = 
                            function(value)
                                eventsDB["self"]["incoming"]["SWING_DAMAGE"]["track"] = value
                            end,
                    },
                    trackFailed = {
                        name = "Track Failed",
                        desc = "Track failed swings",
                        type = "boolean",
                        get = function() end,
                        set = function() end,
                    },
                    normal = {
                        name = "Normal",
                        desc = "Swing hits",
                        type = "group",
                        order = 1,
                        args = {
                            ["color"] = {
                                name = "Color",
                                desc = "Incoming swing color",
                                type = "color",
                                get = 
                                    function() 
                                        return XLog:GetColor(eventsDB["self"]["incoming"]["SWING_DAMAGE"]["normal"]["color"])
                                    end,
                                set = 
                                    function(r,g,b)
                                        local db = eventsDB["self"]["incoming"]["SWING_DAMAGE"]["normal"]["color"]
                                        db["R"], db["G"], db["B"] = r, g, b
                                    end
                            },
                            ["text"] = {
                                name = "Text",
                                desc = "Incoming swing text",
                                type = "string",
                                usage = "<string>",
                                get = 
                                    function() 
                                        return eventsDB["self"]["incoming"]["SWING_DAMAGE"]["normal"]["text"]
                                    end,
                                set = 
                                    function(value)
                                        eventsDB["self"]["incoming"]["SWING_DAMAGE"]["normal"]["text"] = value
                                    end,
                            },
                            ["tab"] = {
                                name = "Chat Tab",
                                desc = "Chat tab to display messages in.",
                                type = 'choice',
                                choices = tabList,
                                get = function(key)
                                    return eventsDB["self"]["incoming"]["SWING_DAMAGE"]["normal"]["index"]
                                end,
                                set = function(key)
                                    eventsDB["self"]["incoming"]["SWING_DAMAGE"]["normal"]["index"] = key
                                end,
                            }
                        }
                    },
                    crits = {
                        name = "Critical Strikes",
                        desc = "Critical Strikes",
                        type = "group",
                        hidden = false,
                        args = {
                            ["color"] = {
                                name = "Color",
                                desc = "Incoming critical swing color",
                                type = "color",
                                get = 
                                    function() 
                                        return XLog:GetColor(eventsDB["self"]["incoming"]["SWING_DAMAGE"]["crit"]["color"])
                                    end,
                                set = 
                                    function(r, g, b) 
                                        local db = eventsDB["self"]["incoming"]["SWING_DAMAGE"]["crit"]["color"]
                                        db["R"], db["G"], db["B"] = r, g, b
                                    end
                            },
                            ["text"] = {
                                name = "Text",
                                desc = "Incoming critical swing text",
                                type = "string",
                                usage = "<string>",
                                get = 
                                    function() 
                                        return eventsDB["self"]["incoming"]["SWING_DAMAGE"]["crit"]["text"]
                                    end,
                                set = 
                                    function(value)
                                        eventsDB["self"]["incoming"]["SWING_DAMAGE"]["crit"]["text"] = value
                                    end,
                            },
                            ["tab"] = {
                                name = "Chat Tab",
                                desc = "Chat tab to display messages in.",
                                type = 'choice',
                                choices = tabList,
                                get = function(key)
                                    return eventsDB["self"]["incoming"]["SWING_DAMAGE"]["crit"]["index"]
                                end,
                                set = function(key)
                                    eventsDB["self"]["incoming"]["SWING_DAMAGE"]["crit"]["index"] = key
                                end,
                            }
                        }
                    },
                    miss = {
                        name = "Failed Attacks",
                        desc = "Failed to hit swings",
                        type = "group",
                        hidden = false,
                        args = {
                            ["color"] = {
                                name = "Color",
                                desc = "Incoming failed swing color",
                                type = "color",
                                get = 
                                    function() 
                                        return XLog:GetColor(eventsDB["self"]["incoming"]["SWING_MISSED"]["normal"]["color"])
                                    end,
                                set = 
                                    function(r, g, b)
                                        local db = eventsDB["self"]["incoming"]["SWING_MISSED"]["normal"]["color"]
                                        db["R"], db["G"], db["B"] = r, g, b
                                    end
                            },
                            ["text"] = {
                                name = "Text",
                                desc = "Incoming failed swing text",
                                type = "string",
                                usage = "<string>",
                                get = 
                                    function() 
                                        return eventsDB["self"]["incoming"]["SWING_MISSED"]["normal"]["text"]
                                    end,
                                set = 
                                    function(value)
                                        eventsDB["self"]["incoming"]["SWING_MISSED"]["normal"]["text"] = value
                                    end,
                            },
                            ["tab"] = {
                                name = "Chat Tab",
                                desc = "Chat tab to display messages in.",
                                type = 'choice',
                                choices = tabList,
                                get = function(key)
                                    return eventsDB["self"]["incoming"]["SWING_MISSED"]["normal"]["index"]
                                end,
                                set = function(key)
                                    eventsDB["self"]["incoming"]["SWING_MISSED"]["normal"]["index"] = key
                                end,
                            }                           
                        }
                    }                   
                }
            },
            spells = {
                name = "Spells",
                desc = "Spell/Special damage",
                type = "group",
                args = {
                    trackHits = {
                        name = "Track Hits",
                        desc = "Track spell hits",
                        type = "boolean",
                        get = 
                            function() 
                                return eventsDB["self"]["incoming"]["SPELL_DAMAGE"]["track"]
                            end,
                        set = 
                            function(value)
                                eventsDB["self"]["incoming"]["SPELL_DAMAGE"]["track"] = value
                            end,
                    },
                    trackFailed = {
                        name = "Track Failed",
                        desc = "Track failed spells",
                        type = "boolean",
                        get = function() end,
                        set = function() end,
                    },
                    normal = {
                        name = "Normal",
                        desc = "Spell hits",
                        type = "group",
                        order = 1,
                        args = {
                            ["color"] = {
                                name = "Color",
                                desc = "Incoming spell color",
                                type = "color",
                                get = 
                                    function() 
                                        return XLog:GetColor(eventsDB["self"]["incoming"]["SPELL_DAMAGE"]["normal"]["color"])
                                    end,
                                set = 
                                    function(r,g,b)
                                        local db = eventsDB["self"]["incoming"]["SPELL_DAMAGE"]["normal"]["color"]
                                        db["R"], db["G"], db["B"] = r, g, b
                                    end
                            },
                            ["text"] = {
                                name = "Text",
                                desc = "Incoming spell text",
                                type = "string",
                                usage = "<string>",
                                get = 
                                    function() 
                                        return eventsDB["self"]["incoming"]["SPELL_DAMAGE"]["normal"]["text"]
                                    end,
                                set = 
                                    function(value)
                                        eventsDB["self"]["incoming"]["SPELL_DAMAGE"]["normal"]["text"] = value
                                    end,
                            },
                            ["tab"] = {
                                name = "Chat Tab",
                                desc = "Chat tab to display messages in.",
                                type = 'choice',
                                choices = tabList,
                                get = function(key)
                                    return eventsDB["self"]["incoming"]["SPELL_DAMAGE"]["normal"]["index"]
                                end,
                                set = function(key)
                                    eventsDB["self"]["incoming"]["SPELL_DAMAGE"]["normal"]["index"] = key
                                end,
                            }
                        }
                    },
                    crits = {
                        name = "Critical Strikes",
                        desc = "Critical Strikes",
                        type = "group",
                        hidden = false,
                        args = {
                            ["color"] = {
                                name = "Color",
                                desc = "Incoming critical swing color",
                                type = "color",
                                get = 
                                    function() 
                                        return XLog:GetColor(eventsDB["self"]["incoming"]["SPELL_DAMAGE"]["crit"]["color"])
                                    end,
                                set = 
                                    function(r, g, b) 
                                        local db = eventsDB["self"]["incoming"]["SPELL_DAMAGE"]["crit"]["color"]
                                        db["R"], db["G"], db["B"] = r, g, b
                                    end
                            },
                            ["text"] = {
                                name = "Text",
                                desc = "Incoming critical swing text",
                                type = "string",
                                usage = "<string>",
                                get = 
                                    function() 
                                        return eventsDB["self"]["incoming"]["SPELL_DAMAGE"]["crit"]["text"]
                                    end,
                                set = 
                                    function(value)
                                        eventsDB["self"]["incoming"]["SPELL_DAMAGE"]["crit"]["text"] = value
                                    end,
                            },
                            ["tab"] = {
                                name = "Chat Tab",
                                desc = "Chat tab to display messages in.",
                                type = 'choice',
                                choices = tabList,
                                get = function(key)
                                    return eventsDB["self"]["incoming"]["SPELL_DAMAGE"]["crit"]["index"]
                                end,
                                set = function(key)
                                    eventsDB["self"]["incoming"]["SPELL_DAMAGE"]["crit"]["index"] = key
                                end,
                            }
                        }
                    },
                    miss = {
                        name = "Failed Attacks",
                        desc = "Failed to hit spells",
                        type = "group",
                        hidden = false,
                        args = {
                            ["color"] = {
                                name = "Color",
                                desc = "Incoming failed spell color",
                                type = "color",
                                get = 
                                    function() 
                                        return XLog:GetColor(eventsDB["self"]["incoming"]["SPELL_MISSED"]["normal"]["color"])
                                    end,
                                set = 
                                    function(r, g, b)
                                        local db = eventsDB["self"]["incoming"]["SPELL_MISSED"]["normal"]["color"]
                                        db["R"], db["G"], db["B"] = r, g, b
                                    end
                            },
                            ["text"] = {
                                name = "Text",
                                desc = "Incoming failed spell text",
                                type = "string",
                                usage = "<string>",
                                get = 
                                    function() 
                                        return eventsDB["self"]["incoming"]["SPELL_MISSED"]["normal"]["text"]
                                    end,
                                set = 
                                    function(value)
                                        eventsDB["self"]["incoming"]["SPELL_MISSED"]["normal"]["text"] = value
                                    end,
                            },
                            ["tab"] = {
                                name = "Chat Tab",
                                desc = "Chat tab to display messages in.",
                                type = 'choice',
                                choices = tabList,
                                get = function(key)
                                    return eventsDB["self"]["incoming"]["SPELL_MISSED"]["normal"]["index"]
                                end,
                                set = function(key)
                                    eventsDB["self"]["incoming"]["SPELL_MISSED"]["normal"]["index"] = key
                                end,
                            }                           
                        }
                    }                   
                }
            }                       
        }
    }

    local general = {
        name = "General",
        desc = "General Options",
        type = "group",
        order = 1,
        args = {
            defaultTab = {
                name = "Chat Tab",
                desc = "Default chat tab to display messages in.",
                type = 'choice',
                choices = rootTabList,
                get = function(key)
                    return eventsDB["defaults"]["index"]
                end,
                set = function(key)
                    eventsDB["defaults"]["index"] = key
                end
            },
            combat = {
            	name = "Combat State",
            	desc = "Settings for combat state changes.",
            	type = "group",
            	args = {
            		track = {
            			name = "Track State Changes",
            			desc = "Enable/disable combat state events",
            			type = "boolean",
            			get = function()
            					return eventsDB["combat"]["track"]
            				end,
            			set = function(value)
            					eventsDB["combat"]["track"] = value
            				end,
            		},
            		start = {
            			name = "Enter",
            			desc = "Settings for when combat starts",
            			type = "group",
            			order = 1,
            			args = {
							["color"] = {
                                name = "Color",
                                desc = "Combat start color",
                                type = "color",
                                get = 
                                    function() 
                                        return XLog:GetColor(eventsDB["combat"]["ENTER_COMBAT"]["color"])
                                    end,
                                set = 
                                    function(r, g, b)
                                        local db = eventsDB["combat"]["ENTER_COMBAT"]["color"]
                                        db["R"], db["G"], db["B"] = r, g, b
                                    end
                            },
                            ["text"] = {
                                name = "Text",
                                desc = "Combat start text",
                                type = "string",
                                usage = "<string>",
                                get = 
                                    function() 
                                        return eventsDB["combat"]["ENTER_COMBAT"]["text"]
                                    end,
                                set = 
                                    function(value)
                                        eventsDB["combat"]["ENTER_COMBAT"]["text"] = value
                                    end,
                            },
                            ["tab"] = {
                                name = "Chat Tab",
                                desc = "Chat tab to display messages in.",
                                type = 'choice',
                                choices = tabList,
                                get = function(key)
                                    return eventsDB["combat"]["ENTER_COMBAT"]["index"]
                                end,
                                set = function(key)
                                    eventsDB["combat"]["ENTER_COMBAT"]["index"] = key
                                end,
                            }
						}
					},
            		["end"] = {
            			name = "Exit",
            			desc = "Settings for when combat ends",
            			type = "group",
            			order = 1,
            			args = {
							["color"] = {
                                name = "Color",
                                desc = "Combat end color",
                                type = "color",
                                get = 
                                    function() 
                                        return XLog:GetColor(eventsDB["combat"]["EXIT_COMBAT"]["color"])
                                    end,
                                set = 
                                    function(r, g, b)
                                        local db = eventsDB["combat"]["EXIT_COMBAT"]["color"]
                                        db["R"], db["G"], db["B"] = r, g, b
                                    end
                            },
                            ["text"] = {
                                name = "Text",
                                desc = "Combat end text",
                                type = "string",
                                usage = "<string>",
                                get = 
                                    function() 
                                        return eventsDB["combat"]["EXIT_COMBAT"]["text"]
                                    end,
                                set = 
                                    function(value)
                                        eventsDB["combat"]["EXIT_COMBAT"]["text"] = value
                                    end,
                            },
                            ["tab"] = {
                                name = "Chat Tab",
                                desc = "Chat tab to display messages in.",
                                type = 'choice',
                                choices = tabList,
                                get = function(key)
                                    return eventsDB["combat"]["EXIT_COMBAT"]["index"]
                                end,
                                set = function(key)
                                    eventsDB["combat"]["EXIT_COMBAT"]["index"] = key
                                end,
                            }
						}
					},
				}				
			},            					
            damageTypes = {
                name = "Spell Schools",
                desc = "Output text for spell schools.",
                type = "group",
                args = {
                    physical = {
                        name = "Physical",
                        desc = "Physical damage text",
                        type = "text",
                        usage = "<string>",
                        get = function()
                                return self.db.profile["settings"]["spellSchools"][1]
                            end,
                        set = function(value)
                                self.db.profile["settings"]["spellSchools"][1] = value
                            end,
                    },
                    holy = {
                        name = "Holy",
                        desc = "Holy damage text",
                        type = "text",
                        usage = "<string>",
                        get = function()
                                return self.db.profile["settings"]["spellSchools"][2]
                            end,
                        set = function(value)
                                self.db.profile["settings"]["spellSchools"][2] = value
                            end,
                    },
                    fire = {
                        name = "Fire",
                        desc = "Fire damage text",
                        type = "text",
                        usage = "<string>",
                        get = function()
                                return self.db.profile["settings"]["spellSchools"][4]
                            end,
                        set = function(value)
                                self.db.profile["settings"]["spellSchools"][4] = value
                            end,
                    },
                    nature = {
                        name = "Nature",
                        desc = "Nature damage text",
                        type = "text",
                        usage = "<string>",
                        get = function()
                                return self.db.profile["settings"]["spellSchools"][8]
                            end,
                        set = function(value)
                                self.db.profile["settings"]["spellSchools"][8] = value
                            end,
                    },
                    frost = {
                        name = "Frost",
                        desc = "Frost damage text",
                        type = "text",
                        usage = "<string>",
                        get = function()
                                return self.db.profile["settings"]["spellSchools"][16]
                            end,
                        set = function(value)
                                self.db.profile["settings"]["spellSchools"][16] = value
                            end,
                    },
                    shadow = {
                        name = "Shadow",
                        desc = "Shadow damage text",
                        type = "text",
                        usage = "<string>",
                        get = function()
                                return self.db.profile["settings"]["spellSchools"][32]
                            end,
                        set = function(value)
                                self.db.profile["settings"]["spellSchools"][32] = value
                            end,
                    },
                    arcane = {
                        name = "Arcane",
                        desc = "Arcane damage text",
                        type = "text",
                        usage = "<string>",
                        get = function()
                                return self.db.profile["settings"]["spellSchools"][64]
                            end,
                        set = function(value)
                                self.db.profile["settings"]["spellSchools"][64] = value
                            end,
                    },
                }
            },          
            optErrs = {
                name = "Error Log",
                desc = "List of errors that may have occured",
                type = "group",
                hidden = false,
                args = {
                    ["missing"] = {
                        name = "Missing Events",
                        desc = "Events that did not have a settings tree",
                        type = "execute",
                        buttonText = "Show Missing Events",
                        func = 
                            function()
                                message("... not yet implemented ...")
                            end,
                    },
                },
            },
        }
    }


    local events = {
        name = "Events",
        desc = "Event Options",
        type = "group",
        order = 2,
        args = {
            outgoing,
            incoming,
        }
    }       

    XLog:AddOption("General", general)
    --XLog:AddOption("Outgoing", outgoing)
    --XLog:AddOption("Incoming", incoming)
    XLog:AddOption("Events", events)

end

function GetIndex(tree)
  return tree["index"]
end

function ChatName(index)
  local win = GetChatWindowInfo(index)
  if strlen(win) > 0 then
    return index .. ". " .. win
  else
    return nil
  end
end

