local MAJOR_VERSION = "LibXEvent-1.0"
local MINOR_VERSION = 0001

--local MyHandler = {}

local LibXEvent = Rock:NewLibrary(MAJOR_VERSION, MINOR_VERSION)
--local LibXEvent = {}
local self = LibXEvent

local selfPlayerName = ""


LibXEvent.spellSchools = {
	[0] = "None",
	[1] = "Physical",
	[2] = "Holy",
	[4] = "Fire",
	[8] = "Nature",
	[16] = "Frost",
	[32] = "Shadow",
	[64] = "Arcane",
}

LibXEvent.powerTypes = {
	[0] = "mana",
	[1] = "rage",
	[2] = "focus",
	[3] = "energy",
	[4] = "pethappiness",
	[5] = "runes",
	[-2] = "health"
}


-- general event function

function LibXEvent:OnEvent(event, ...)

  --ChatFrame1:AddMessage(arg1 .. arg2 .. arg3)

  LibXEvent.eventFunctions[event](_,select(1,...))
  --ChatFrame1:AddMessage("received " .. event)

end



--*************************************
-- Handlers
-- Handlers will be called after an event is parsed (if there are any)

LibXEvent.handlerCount = 0
LibXEvent.handlers = {}

-- Register a handler
function LibXEvent:RegisterHandler(func)

	if type(func) == "function" then
		LibXEvent.handlerCount = LibXEvent.handlerCount + 1
		LibXEvent.handlers[LibXEvent.handlerCount] = func
		
		--print("Successfully registered handler")
	else
		--print("Not a function")
	end	

end

-- invoke handlers
function LibXEvent:InvokeHandlers(...)
	for k,_ in ipairs(LibXEvent.handlers) do
		-- call each registered function in turn
		LibXEvent.handlers[k](_,select(1, ...))
	end
end


-- End Handler functions
--********************************************



-- Combat Log Event
function LibXEvent:CombatLogEvent(...)
	
	-- don't bother if we have no handlers
	if LibXEvent.handlerCount > 0 then
		-- Parse to eventInfo	
		local info = {}
		info = LibXEvent:ParseEvent(select(1, ...))
		
		-- pass to handlers
		LibXEvent:InvokeHandlers(info)
	end
	
end

-- Combat Starts event
function LibXEvent:PLAYER_REGEN_DISABLED()

	if LibXEvent.handlerCount > 0 then
		-- build info table
		local info = {
			event = "PLAYER_REGEN_DISABLED",
			source = selfPlayerName,
			self = true,		
		}
		
		LibXEvent:InvokeHandlers(info)
	end

end

-- Combat Ends event
function LibXEvent:PLAYER_REGEN_ENABLED()

	if LibXEvent.handlerCount > 0 then
		-- build info table
		local info = {
			event = "PLAYER_REGEN_ENABLED",
			source = selfPlayerName,
			self = true,		
		}
		
		LibXEvent:InvokeHandlers(info)
	end

end

local COMBATLOG_HONORGAIN = "%s dies, honorable kill Rank : %s (%d Honor Points)"

function LibXEvent:CHAT_MSG_COMBAT_HONOR_GAIN(eventString)	

	if LibXEvent.handlerCount > 0 then
		local pattern = LibXEvent:MakePattern(COMBATLOG_HONORGAIN)
		local info = {}
		
		-- parse string
		_, _, info.sourceNameServer, info.rank, info.honor = string.find(eventString, pattern)
		
		-- build extra info
		info.sourceName = LibXEvent:StripServer(info.sourceNameServer)
		info.event = "CHAT_MSG_COMBAT_HONOR_GAIN"
		info.self = true
		
		LibXEvent:InvokeHandlers(info)
	end
		
end



--*********************************************
-- Event Parsing
--*********************************************

function LibXEvent:ParseEvent(...)

	-- info is passed from function to function, as its much faster
	-- to pass references to start than to join tables later
	local info = {}
	
	info.timestamp, info.event, info.sourceGUID, info.sourceName, info.sourceFlags, info.destGUID, info.destName, info.destFlags = select(1, ...)

	-- parse prefix and suffix	
	local _, _, prefix, suffix = string.find(info.event, "(.-)_(.+)")
		
	-- call parser for args 9 and up
	info = LibXEvent:DoParse(info, prefix, suffix, select(9, ...))
	
	
	--ChatFrame1:AddMessage("after" .. info.event)
	
	-- is the player involved?
	if (info.sourceName == selfPlayerName) then
		info.self, info.direction = true, "outgoing"
	elseif (info.destName == selfPlayerName) then
		info.self, info.direction = true, "incoming"
	else
		info.self, info.direction = false, "other"
	end
	
	-- clean up names
	info.sourceNameServer = info.sourceName			-- name with server (PvP)
	info.sourceName = LibXEvent:StripServer(info.sourceName) 	-- name without server (PvP)
	info.destNameServer = info.destName				-- name with server (PvP)
	info.destName = LibXEvent:StripServer(info.destName)		-- name without server (PvP)
		
	return info
end

function LibXEvent:StripServer(name)

	if (name ~= nil) then
	  	if string.find(name, "-", 1) then
	  		name, server = strsplit("-", name)
	  	end
 	end
	
	return name

end


function LibXEvent:MakePattern(const)
	-- escape brackets
	local newPattern = string.gsub(const, "%(", "%%(")
	newPattern = string.gsub(newPattern, "%)", "%%)")
	
	-- swap %d and %s for .+
	newPattern = string.gsub(newPattern, "(%%s)", "(.+)")
	newPattern = string.gsub(newPattern, "(%%d)", "(.+)")
	
	return newPattern
end


-- **************************************
-- Main Parser

function LibXEvent:DoParse(info, prefix, suffix, ...)
		
	local pf = LibXEvent.prefixFunctions[prefix]
	local sf = LibXEvent.suffixFunctions[suffix]

	--local info = {}
	
	if (pf ~= nil and sf ~= nil) then
		--local suffixIndex
		-- take note of this part...if I don't pass a nil as the first param,
		-- I lose my first param...WHY?
		-- don't forget this change if some weirdness occurs in-game.		
		info, suffixIndex = pf(_, info, select(1, ...))
		info = sf(_, info, select(suffixIndex, ...))
		
	end
	
	return info
end




-- *******************************************
-- Prefix Parsers
-- *******************************************

function LibXEvent:PrefixSwing(info, ...)
	--local info = {}
	local suffixIndex = 1
	 
	return info, suffixIndex
end

function LibXEvent:PrefixRange(info, ...)
	--local info = {}
	local suffixIndex = 4
	 
	info.spellID, info.spellName, info.spellSchool = select(1, ...)
	--info.extraSpellSchool = LibXEvent.spellSchools[info.extraSpellSchool]
	 
	return info, suffixIndex
end

-- Tested: A select(1,...) call is approximately 2% faster than using parameters
function LibXEvent:PrefixSpell(info, ...)
	--print("PrefixSpell" .. info.event)
	local suffixIndex = 4

	info.spellID, info.spellName, info.spellSchool = select(1, ...)
	--info.spellSchool = LibXEvent.spellSchools[info.spellSchool]
			 
	return info, suffixIndex
end

function LibXEvent:PrefixEnviro(...)

end


--****************
-- Suffixes

-- Tested: its approximately 7% faster to put the school lookup in here when needed
-- rather than a nil blanket "if not nil then fixup" check in the entry method.
function LibXEvent:SuffixDamage(info, ...)

	info.amount, info.school, info.resisted, info.blocked, info.absorbed, info.critical, info.glancing, info.crushing = select(1, ...)	
	--info.school = LibXEvent.spellSchools[info.school]
	
	return info
end

function LibXEvent:SuffixMissed(info, ...)
	--local info = {}
	info.missType = select(1, ...)
	
	return info
end

function LibXEvent:SuffixHeal(info, ...)
	--local info = {}
	info.amount, info.critical = select(1, ...)
	
	return info
end

function LibXEvent:SuffixEnergize(info, ...)
	--local info = {}
	
	info.amount, info.powerType = select(1, ...)
	--info.powerType = LibXEvent.powerTypes[info.powerType]
	
	return info
end

function LibXEvent:SuffixDrain(info, ...)
	--local info = {}
	
	info.amount, info.powerType, info.extraAmount = select(1, ...)
	--info.powerType = LibXEvent.powerTypes[info.powerType]
	
	return info
end

function LibXEvent:SuffixLeech(info, ...)
	--local info = {}
	
	info.amount, info.powerType, info.extraAmount = select(1, ...)
	--info.powerType = LibXEvent.powerTypes[info.powerType]
	
	return info
end

function LibXEvent:SuffixInterrupt(info, ...)
	--local info = {}
	
	info.extraSpellID, info.extraSpellName, info.extraSpellSchool = select(1, ...)
	--info.extraSpellSchool = LibXEvent.spellSchools[info.extraSpellSchool]
	
	return info
end

function LibXEvent:SuffixDispelFailed(info, ...)
	--local info = {}
	
	info.extraSpellID, info.extraSpellName, info.extraSpellSchool = select(1, ...)
	--info.extraSpellSchool = LibXEvent.spellSchools[info.extraSpellSchool]
	
	return info
end

function LibXEvent:SuffixAuraDispelled(...)
	--local info = {}
	
	info.extraSpellID, info.extraSpellName, info.extraSpellSchool, info.auraType = select(1, ...)
	--info.extraSpellSchool = LibXEvent.spellSchools[info.extraSpellSchool]
	
	return info
end

function LibXEvent:SuffixAuraStolen(info, ...)
	--local info = {}
	
	info.extraSpellID, info.extraSpellName, info.extraSpellSchool = select(1, ...)
	--info.extraSpellSchool = LibXEvent.spellSchools[info.extraSpellSchool]
	
	return info
end

function LibXEvent:SuffixExtraAttacks(info, ...)
	--local info = {}
	info.amount = select(1, ...)
	
	return info
end

function LibXEvent:SuffixAuraApplied(info, ...)
	--local info = {}
	info.auraType = select(1, ...)
		
	return info
end

function LibXEvent:SuffixAuraRemoved(info, ...)
	--local info = {}
	info.auraType = select(1, ...)
	
	return info
end

function LibXEvent:SuffixAuraAppliedDose(info, ...)
	--local info = {}
	info.auraType, info.amount = select(1, ...)
	
	return info
end

function LibXEvent:SuffixAuraRemovedDose(info, ...)
	--local info = {}
	info.auraType, info.amount = select(1, ...)
	
	return info
end

function LibXEvent:SuffixCastStart(info, ...)
	--local info = {}
	-- no args
	
	return info
end

function LibXEvent:SuffixCastSuccess(info, ...)
	--local info = {}
	-- no args
	
	return info
end

function LibXEvent:SuffixCastFailed(info, ...)
	--local info = {}
	info.failedType = select(1, ...)
	
	return info
end

function LibXEvent:SuffixInstaKill(info, ...)
	--local info = {}
	-- no args
	
	return info
end

function LibXEvent:SuffixDurabilityDamage(info, ...)
	--local info = {}
	-- no args
	
	return info
end

function LibXEvent:SuffixDurabilityDamageAll(info, ...)
	--local info = {}
	-- no args
	
	return info
end






LibXEvent.prefixFunctions = {
	["SWING"] = LibXEvent.PrefixSwing,
	["RANGE"] = LibXEvent.PrefixRange,
	["SPELL"] = LibXEvent.PrefixSpell,
	--["ENVIRONMENTAL"] = LibXEvent.PrefixEnviro,
	
	-- special events
	["DAMAGE"] = LibXEvent.PrefixSpell,
	
}

LibXEvent.suffixFunctions = {
	["DAMAGE"] = LibXEvent.SuffixDamage,
	["PERIODIC_DAMAGE"] = LibXEvent.SuffixDamage,
	["MISSED"] = LibXEvent.SuffixMissed,
	["HEAL"] = LibXEvent.SuffixHeal,
	["PERIODIC_HEAL"] = LibXEvent.SuffixHeal,
	["ENERGIZE"] = LibXEvent.SuffixEnergize,
	["DRAIN"] = LibXEvent.SuffixDrain,
	["LEECH"] = LibXEvent.SuffixLeech,
	["INTERRUPT"] = LibXEvent.SuffixInterrupt,
	["DISPEL_FAILED"] = LibXEvent.SuffixDispelFailed,
	["AURA_DISPELLED"] = LibXEvent.SuffixAuraDispelled,
	["AURA_STOLEN"] = LibXEvent.SuffixAuraStolen,
	["EXTRA_ATTACKS"] = LibXEvent.SuffixExtraAttacks,
	["AURA_APPLIED"] = LibXEvent.SuffixAuraApplied,
	["AURA_REMOVED"] = LibXEvent.SuffixAuraRemoved,
	["AURA_APPLIED_DOSE"] = LibXEvent.SuffixAuraAppliedDose,
	["AURA_REMOVED_DOSE"] = LibXEvent.SuffixAuraRemovedDose,
	["CAST_START"] = LibXEvent.SuffixCastStart,
	["CAST_SUCCESS"] = LibXEvent.SuffixCastSuccess,
	["CAST_FAILED"] = LibXEvent.SuffixCastFailed,
	["INSTAKILL"] = LibXEvent.SuffixInstaKill,
	["DURABILITY_DAMAGE"] = LibXEvent.SuffixDurabilityDamage,
	["DURABILITY_DAMAGE_ALL"] = LibXEvent.SuffixDurabilityDamageAll,
	
	-- special events
	["SHIELD"] = LibXEvent.SuffixDamage,
	["SPLIT"] = LibXEvent.SuffixDamage,
	["SHIELD_MISSED"] = LibXEvent.SuffixMissed
}

LibXEvent.eventFunctions = {
  ["COMBAT_LOG_EVENT_UNFILTERED"] = LibXEvent.CombatLogEvent,
  ["PLAYER_REGEN_DISABLED"] = LibXEvent.PLAYER_REGEN_DISABLED,
  ["PLAYER_REGEN_ENABLED"] = LibXEvent.PLAYER_REGEN_ENABLED,
	["CHAT_MSG_COMBAT_HONOR_GAIN"] = LibXEvent.CHAT_MSG_COMBAT_HONOR_GAIN,
}
	



--*************************************************************
-- Debugging

--[[ test handler, different namespace
function MyHandler:Handler(info)
	
	--print(info)
	--print("<" .. info.amount .. "> " .. info.school)
	LibXEvent:PrintTable(info)

end
]]


-- debug
function LibXEvent:DebugStart()
	print("DEBUG START")
	
	selfPlayerName = "Crazyoldjoe"
	
	local timestamp = 1234
	local event = "SPELL_DAMAGE"
	local sourceGUID = nil
	local sourceName = "Xyza"
	local sourceFlags = nil
	local destGUID = nil
	local destName = selfPlayerName
	local destFlags = nil
	
	-- register handler
	LibXEvent:RegisterHandler(MyHandler.Handler)
	
	local start = os.clock()
	
	local info = {}
	
	
	for i=1, 1, 1 do
		local amount = math.random(50,1000)
		--LibXEvent:PLAYER_REGEN_ENABLED()
		--LibXEvent:CombatLogEvent("",timestamp, event, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, -1, "Fireball", 4, amount, 4, _, _, _, "yes")
		--LibXEvent:CHAT_MSG_COMBAT_HONOR_GAIN("Someplayer dies, honorable kill Rank : Scout (10 Honor Points)")
	end
	
	--local info = ParseEvent(timestamp, event, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, -1, "Healing Wave", "Nature", 1000, "no")
	
	printf(string.format("elapsed time: %.8f\n", os.clock() - start))
	
	--LibXEvent:PrintTable(info)
end

function LibXEvent:PrintTable(table)
	printf("Table Info:")
	
	for key, value in pairs(table) do
		printf(string.format(" %s=%s", key, LibXEvent:ToString(value)))
	end
end

function LibXEvent:ToString(value)
  if type(value) == "boolean" then
    if value then
      value = "True"
    else
      value = "False"
    end
  end
  
  if type(value) == "table" then
  	value = "table"
  end
  
  value = string.format("%s", value or "nil")
  
  return value
end

function printf(msg)
	--ChatFrame1:AddMessage(string.format("%s", msg or "nil"))
	--print(string.format("%s", msg or "nil"))
end

-- End debugging code
--*******************************************************************


--LibXEvent:DebugStart()

--LibXEvent:OnEnable()

-- startup code
selfPlayerName = UnitName("player")

local frame = CreateFrame("Frame", "LibXEventFrame")
frame:SetScript("OnEvent", LibXEvent.OnEvent)
frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

--LibXEvent:AddEventListener("Blizzard", "COMBAT_LOG_EVENT_UNFILTERED", LibXEvent.CombatLogEvent)
	--LibXEvent:AddEventListener("Blizzard", "PLAYER_REGEN_DISABLED", LibXEvent.PLAYER_REGEN_DISABLED)
	--LibXEvent:AddEventListener("Blizzard", "PLAYER_REGEN_ENABLED", LibXEvent.PLAYER_REGEN_ENABLED)
	--LibXEvent:AddEventListener("Blizzard", "CHAT_MSG_COMBAT_HONOR_GAIN", LibXEvent.CHAT_MSG_COMBAT_HONOR_GAIN)
	

-- end startup code


LibXEvent:SetExportedMethods("RegisterHandler")

Rock:FinalizeLibrary(MAJOR_VERSION)